package t::CorrectPod;

use strict;
use warnings;

=head1 DESCRIPTION

This file has no naked sub and all subs have correct syntax

=cut

=head2 foo

this is a description for the foo sub

=cut

sub foo { }

=head2 bar

this is a description for the bar sub

=cut

sub bar { }

=head2 baz

this is a description for the baz sub

=cut

sub baz { }

1;
